﻿using System;
using System.Collections.Generic;
using System.IO;

namespace PythonUtils
{
    public class NDArray
    {
        Dictionary<string, List<double>> _data = new Dictionary<string, List<double>>();
        Dictionary<int, string> _headerIndex = new Dictionary<int, string>();

        private NDArray()
        {
        }

        public bool HasHeader
        {
            get { return _data.Keys.Count != 0; }
        }

        public double [,] ToArray2D()
        {
            if (_headerIndex.Keys.Count != 2)
            {
                throw new InvalidOperationException("Not 2-Dim array");
            }

            double[,] data = new double[_data.Keys.Count, _data[_headerIndex[0]].Count];

            string header1st = _headerIndex[0];
            string header2nd = _headerIndex[1];

            for (int dataIdx = 0; dataIdx < data.GetLength(1); dataIdx ++)
            {
                data[0, dataIdx] = _data[header1st][dataIdx];
                data[1, dataIdx] = _data[header2nd][dataIdx];
            }

            return data;
        }

        void AddHeader(string[] headers)
        {
            int index = 0;

            foreach (string header in headers)
            {
                if (string.IsNullOrEmpty(header) == false)
                {
                    _data[header] = new List<double>();
                    _headerIndex[index++] = header;
                }
            }
        }

        void AddData(string[] values)
        {
            for (int i = 0; i < values.Length; i ++)
            {
                string value = values[i];

                if (double.TryParse(value, out double result) == true)
                {
                    _data[_headerIndex[i]].Add(result);
                }
                else
                {
                    _data[_headerIndex[i]].Add(0);
                }
            }
        }

        public static NDArray LoadFrom(string filePath, char delimiter = ',', int skipRows = 1)
        {
            NDArray data = new NDArray();

            foreach (string line in File.ReadAllLines(filePath))
            {
                if (skipRows >= 1)
                {
                    if (data.HasHeader == false)
                    {
                        string[] headers = line.Split(delimiter);
                        data.AddHeader(headers);
                    }

                    skipRows--;
                    continue;
                }

                string[] values = line.Split(delimiter);
                data.AddData(values);
            }

            return data;
        }
    }
}
